use Test::More;
use Test::MockObject;
use Test::MockObject::Extends;
use DE_EPAGES::Core::API::PerlTools qw (isValid);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite  );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath ExistsObjectByPath LoadRootObject JoinObjectPath);
use DE_EPAGES::Presentation::Servlet::PageType;

use DE_EPAGES::ELogistics::API::RemoteELogisticsPickPost;
use DE_EPAGES::ELogistics::API::RemoteELogisticsBarcode;
use DE_EPAGES::ELogistics::Hooks::Basket;

use strict;

my $ROOTPATH = 'Test';

sub run_tests {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            # initializsation
            ok( Init(), 'initialization' );
            eval {
                testImport();
                testPickPostService();
                testBarCodeCreation();
                testPickPostAddressStorage();
            };
            if (ExistsError()) {
                my $Error = GetError();
                fail('Error - '. $Error->code);
                diag($Error->longMessage);
                # always cleanup
                ok( Cleanup(), 'cleanup');
                Error($Error);
            }
            # cleanup
            ok( Cleanup(), 'cleanup');
        }
    );
}

sub testImport{
    my $Shop = LoadObjectByPath( JoinObjectPath([$ROOTPATH,'TestShopELogistics']) );
    isa_ok($Shop, 'DE_EPAGES::Shop::API::Object::Shop');

    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/eLogisticsLogistiker');
    isa_ok($ShippingFulfillmentMethod, 'DE_EPAGES::ELogistics::API::Object::ShippingFulfillmentMethodELogistics');

    my $Customer = $Shop->path('Customers/TestCustomer');
    isa_ok($Customer, 'DE_EPAGES::Customer::API::Object::Customer' );

    my $Order = $Shop->path('Customers/TestCustomer/Orders/1234-5678');
    isa_ok($Order, 'DE_EPAGES::Order::API::Object::CustomerOrder' );

    cmp_ok(scalar @{$Order->get('Documents')}, '>=',1, 'number of orderdocuments');

    my $ShippingLabelELogistics = $Shop->path('Customers/TestCustomer/Orders/1234-5678/ShippingLabels/1234-5678-1');
    isa_ok($ShippingLabelELogistics, 'DE_EPAGES::ELogistics::API::Object::ShippingLabelELogistics' );

    my $Transaction = $Shop->path('ShippingOrders/ShippingOrderELogistics');
    isa_ok($Transaction, 'DE_EPAGES::ExternalShipping::API::Object::ShippingTransaction' );

    cmp_ok(scalar @{$Transaction->get('ShippingLabels')}, '>=',1, 'number of labels');

    foreach my $ShippingLabel (@{$Transaction->get('ShippingLabels')}) {
        isa_ok($ShippingLabel, 'DE_EPAGES::ELogistics::API::Object::ShippingLabelELogistics' );
    }
}

sub testPickPostService {
    my $Shop = LoadObjectByPath( JoinObjectPath([$ROOTPATH,'TestShopELogistics']) );
    my $Customer = $Shop->path('Customers/TestCustomer');

    my $Remote = DE_EPAGES::ELogistics::API::RemoteELogisticsPickPost->new();

    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/eLogisticsLogistiker');

    my %hNewShippingAddressData = (
            'Alias'             => 'ELogisticsPickupAddress',
            'FirstName'         => $Customer->get('BillingAddress')->get('FirstName'),
            'LastName'          => $Customer->get('BillingAddress')->get('LastName'),
            'Street'            => 'Postfach 417',
            'Zipcode'           => '8840',
            'City'              => 'Einsiedeln',
            'CountryID'         => 756, #always switzerland at the moment
            'EMail'             => $Customer->get('BillingAddress')->get('EMail'),
            'PhoneCell'         => $Customer->get('BillingAddress')->get('PhoneCell'),
            'IsPickupAddress'   => 1,
    );

    my $hFormValues = { 'EnablePickPostAddressSMS' => 1,
                        'EnablePickPostAddressEmail' => 1};

    my $result = $Remote->CallCustomerPickPostNumber(  $ShippingFulfillmentMethod,
                                                    \%hNewShippingAddressData,
                                                    $hFormValues,
                                                    $Customer->alias,
                                                    1);

    #check if pickpostnumber is returned
    cmp_ok(isValid($result->[0]->{'PickPostNumber'}),'==',1,'pickpost number returned');
}

sub testBarCodeCreation {
    my $Shop = LoadObjectByPath( JoinObjectPath([$ROOTPATH,'TestShopELogistics']) );
    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/eLogisticsLogistiker');
    my $Transaction = $Shop->path('ShippingOrders/ShippingOrderELogistics');

    my $Remote = DE_EPAGES::ELogistics::API::RemoteELogisticsBarcode->new();
    my $result = $Remote->CallGenerateLabelRequest($ShippingFulfillmentMethod,$Transaction,'GIF',1);

    cmp_ok(scalar @{$result}, '==',2, 'number of returned items');
    foreach my $Item ( @{$result} ){
        cmp_ok(isValid( $Item->{'Label'} ),'==',1,'label is returned');
    }
}

sub testPickPostAddressStorage {
    my $hParams;
    $hParams = _initHashParams();
    DE_EPAGES::ELogistics::Hooks::Basket::OnSaveMultiCheckoutShipping($hParams);
    my $Basket = LoadObjectByPath('/Test/TestShopELogistics/Users/mmustermann/Baskets/TestBasket/');

    ok( $Basket->get('ShippingAddress')->get('Street') eq 'Postfach 417', 'the PickPost address is correctly stored');
}

sub _initHashParams {

    my $Form = DE_EPAGES::Presentation::API::Form->new();
    my $mock_Form = Test::MockObject::Extends->new( $Form );
    $mock_Form->mock( 'executeFormError', sub{
        Error('OnSaveMultiCheckoutShippingHookError');
    } );

    my $Basket = LoadObjectByPath('/Test/TestShopELogistics/Users/mmustermann/Baskets/TestBasket');
    my $Servlet = DE_EPAGES::Presentation::Servlet::PageType->new();

    $Servlet->{'Params'} = {
            EnableELogisticPickPostAddress => 'on',
            PickPostAddressStreet => 'Postfach 417',
            PickPostAddressZipCode => '8840',
            PickPostAddressCity => 'Einsiedeln',
            PickPostAddressEmail => 'r.domenech@epages.com',
            PickPostAddressMobile => '',
            EnablePickPostAddressEmail => '1',
            EnablePickPostAddressSMS => '',
    };

    my $mock_servlet = Test::MockObject::Extends->new($Servlet);
    $Servlet->set_always( form => $Form );
    $Servlet->set_always( object => $Basket );
    $Servlet->set_always( languageID => 2 );

    my $hParams = {
        Servlet => $mock_servlet
    };

    return $hParams;
}

sub Main {
    run_tests;
    done_testing;
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $System = LoadRootObject();
    $System->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript( Sub => \&Main );

1;